/*
 * Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */
import { Injectable } from "@angular/core";
import { HttpClient } from "@angular/common/http";
import { Observable } from "rxjs";
import { CapabilityRequest } from "../model/capability-request";
import { CapabilityExecution } from "../model/capability-execution";
import { JsonObject } from "@angular/compiler-cli/ngcc/src/packages/entry_point";
import {CapabilityVersion} from "../model/capability-version";

@Injectable({
  providedIn: "root",
})
export class CapabilityLauncherService {
  private endpoint = "capability/";

  constructor(private httpClient: HttpClient) {}

  /**
   * Create capability request and send it to capability service
   * @param: capabilityName  Name of capability to create request for
   * @param: parameters  Parameters for capability request
   */
  createRequest(capabilityName: string, parameters: JsonObject): Observable<CapabilityRequest> {
    const url = this.endpoint + capabilityName + "/request/create";
    return this.httpClient.post<CapabilityRequest>(url, JSON.stringify({ parameters: parameters }));
  }

  /**
   * Create standard CMS imaging request from previously executed calibration request and send it to capability service
   * @param: capabilityName  Name of capability to create request for
   * @param: requestId Id of calibration request to image
   */
  createImageRequestFromPreviousCal(
    followonType: string,
    selectedVersion: CapabilityVersion,
  ): Observable<CapabilityRequest> {

    const url = this.endpoint + "request/" + selectedVersion.capability_request_id + "/version/"
      + selectedVersion.version_number + "/followon/" + followonType;
    return this.httpClient.post<CapabilityRequest>(url, null);
  }
  /**
   * Submit capability request
   * @param: requestId  ID of capability request to submit
   */
  submit(requestId: string, versionNumber: number): Observable<CapabilityExecution> {
    const url = `${this.endpoint}request/${requestId}/version/${versionNumber}/submit`;
    return this.httpClient.post<CapabilityExecution>(url, null);
  }

  /**
   * Launch calibration using the sdmId of a completed ingestion
   * @param: capabilityName  Name of capability to create request for
   * @param: parameters logData and sdmId
   * example parameters: { "sdmId": string }
   * Now that calibration can be initiated by
   */
  launchCalibrationFromSdmId(
    capabilityName: string,
    parameters: JsonObject,
  ): Observable<any> {
    const url = this.endpoint + capabilityName + "/request/create-and-submit";
    return this.httpClient.post<any>(url, JSON.stringify(parameters));
  }
}
