/*
 * Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */
import { Component, OnInit } from "@angular/core";
import { CapabilityLauncherService } from "./services/capability-launcher.service";
import { CapabilityRequest } from "./model/capability-request";
import { JsonObject } from "@angular/compiler-cli/ngcc/src/packages/entry_point";
import { CapabilityRequestService } from "./services/capability-request.service";

@Component({
  selector: "app-workspaces",
  templateUrl: "./workspaces.component.html",
  styleUrls: ["./workspaces.component.scss"],
})
export class WorkspacesComponent implements OnInit {
  public productLocator: string;
  public calProductLocator: string;
  public userEmail: string;
  public inputFileList: FileList;
  public cmsPath: string;
  public sdmId: string;

  constructor(
    private capabilityLauncher: CapabilityLauncherService,
    private capabilityRequestService: CapabilityRequestService,
  ) {
    // this is the famous 394 MB 13B-014 execblock, a useful smallish test
    this.productLocator = "uid://evla/quicklook_image/f5610635-efe3-49b4-a281-3c0f871e7e5b";
    this.userEmail = null;
  }

  ngOnInit(): void {}

  /**
   * OnClick method that creates a capability request for the null capability and submits it
   */
  nullButtonOnClick(): void {
    this.launchCapability("null", { arguments: "-g" });
  }

  /**
   * OnClick method that creates a capability request for the null capability and submits it to CV-NAASC
   */
  nullCVButtonOnClick(): void {
    this.launchCapability("null", { arguments: "-g", "remote-cv": true });
  }

  /**
   * OnClick method that creates a capability request for the null capability and submits it to CV-NAASC
   */
  nullNMTButtonOnClick(): void {
    this.launchCapability("null", { arguments: "-g", remote: true });
  }

  /**
   * OnClick method that creates a capability request for the null_dag capability
   */
  nullDagButtonOnClick(): void {
    this.launchCapability("null_dag", {});
  }

  /**
   * OnClick method that creates a capability request a given capability and submits it with the standard parameters:
   * - Product locator
   * - User email
    */
  standardLaunchCapabilityOnClick(capabilityName: string): void {
    if (capabilityName === "std_cms_imaging") {
      this.productLocator = undefined;
    }
    this.launchCapability(capabilityName, {
      product_locator: this.productLocator,
      user_email: this.userEmail,
    });
  }

  /**
   * OnClick method that creates a capability request for a imaging capability and submits it with the CMS imaging parameters:
   * - User email
   * - CMS Path
   * - SDM ID
   */
  LaunchImagingCapabilityOnClick(capabilityName: string): void {
    this.launchCapability(capabilityName, {
      user_email: this.userEmail,
      cms_path: this.cmsPath,
      sdmId: this.sdmId,
    });
  }

  /**
   * OnClick method that creates a capability request for a restore capability and submits it with the restore parameters:
   * - EB Product Locator
   * - Calibration Product Locator
   * - User email
   */
  LaunchRestoreCapabilityOnClick(capabilityName: string): void {
    this.launchCapability(capabilityName, {
      product_locator: this.productLocator,
      cal_locator: this.calProductLocator,
      user_email: this.userEmail,
    });
  }

  /**
   * method that sets the user input Science Product Locator for the download capability
   * @param spl the Science Product Locator to download
   */
  setProductLocator(spl: string): void {
    this.productLocator = spl;
  }

  /**
   * method that sets the user input Calibration Science Product Locator
   * @param calSpl the calibration SPL to use
   */
  setCalProductLocator(calSpl: string): void {
    this.calProductLocator = calSpl;
  }

  /**
   * method to set the user email for notifications
   * @param email user specified email
   */
  setUserEmail(email: string): void {
    this.userEmail = email;
  }

  /**
   * Set files to be uploaded to variable
   * @param inputFileList  List of files from modal form
   */
  public setInputFileList(inputFileList: FileList) {
    this.inputFileList = inputFileList;
  }

  /**
   * method to set the CMS Path for Standard Imaging
   * @param path CMS path for imaging
   */
  setCmsPath(path: string): void {
    this.cmsPath = path;
  }

  /**
   * method to set the SDM ID for Standard Imaging
   * @param path SDM ID for imaging
   */
  setSdmId(id: string): void {
    this.sdmId = id;
  }

  /**
   * Method that uses the capabilityLauncher service to launch a capability
   * @param capabilityName  Name of capability
   * @param parameters      Parameters of capability request
   */
  launchCapability(capabilityName: string, parameters: JsonObject): void {
    const createRequestObserver = {
      next: (requestResponse: CapabilityRequest) => {
        if (requestResponse.id) {
          // Capability request created; ID found
          // Redirect to request status page
          if (this.inputFileList) {
            this.capabilityRequestService.addFilesToVersion(requestResponse.id, this.inputFileList);
          }
          this.capabilityRequestService.redirectToRequestStatusPage(requestResponse.id);
        }
      },
      error: (error) => {
        console.log(error);
      },
    };

    // Create capability request
    this.capabilityLauncher
      .createRequest(capabilityName, parameters)
      .subscribe(createRequestObserver);
  }
}
